/* =============================================================================
*                 Texas Instruments PGA970 Platform Software
* (c) Copyright Texas Instruments, Incorporated. All Rights Reserved.
*
* Use of this software is controlled by the terms and conditions found
* in the license agreement under which this software has been supplied.
* =========================================================================== */
/**
* @file pga970_adc.c
*
* This File contains code used for PGA970 ADC related activities.
*
* @path $(CSLPATH)\PGA970\source\chip
*
* @rev 1.1
*/
/* -----------------------------------------------------------------------------
*!
*! Revision History
*! ===================================
*! 29-Apr-2015 mf: Modified TusharB. Added ADC module related functions.
* =========================================================================== */

/*******************************************************************************
* INCLUDE FILES
*******************************************************************************/
#include "pga970_platf.h"

const S2 nGain = 18;
const S2 nOffset = 220;

VUS ADC_Count1;         /* Track 2.048ms time*/

SL demod1;
SL demod2;
SL difference;
SL sum;
SL ratio;
SL vi;
SL ptat;
SL coil3;
US test;
/* ========================================================================== */
/**
 * @fn ADC_Reset_Init() Initializes all ADC related global variables.
 *
 * @see pga970_adc.h
 */
/* ========================================================================== */
void ADC_Reset_Init(void)
{
   ADC_Count1 = 16;
}

/* ========================================================================== */
/**
 * @fn ADC_Config() Configure ADC channels.
 *
 * @see pga970_adc.h
 */
/* ========================================================================== */
void ADC_Config(UC demod1configValue, UC demod2configValue, UC adc3configValue,
                UC s1s2Cfg1, UC s3Cfg1)
{
   S1_S2_DEMOD_CFG_1 = s1s2Cfg1;
   S3_ADC_CFG_1 = s3Cfg1;
   /*Configure the connection of S1, S2 and S3 output*/
   AMUX_CTRL = ((AMUX_CTRL & ~0x0E) |
                  ((S1_OP_TO_ADC1 | S2_OP_TO_ADC2 | S3_OP_TO_ADC3) & 0x0E));
   DEMOD1_CONFIG = demod1configValue;
   DEMOD2_CONFIG = demod2configValue;
   ADC3_CONFIG = adc3configValue;
}

/* ========================================================================== */
/**
 * @fn ADC_Handler() Interrupt subroutine for P or T channel ADC.
 *
 * @see pga970_adc.h
 */
/* ========================================================================== */
interrupt void ADC_Handler(void)
{
    SL coeffP;
    ADC_Count1++;
#if(WATCH_DOG_TIMER_TESTING==1)
    if(WDOG_Low_Time_Cnt1==0)
    {
       WDOG_Low_Time_Cnt1=ADC_Count1;
    }

#endif /*#if(WATCH_DOG_TIMER_TESTING==1)*/
    demod1 = (DEMOD1_DATA >> 8); //read 16bit ADC1 value
    demod2 = (DEMOD2_DATA >> 8); //read 16bit ADC2 value

    difference = demod1 - demod2;
    sum = demod1 + demod2;

    if (sum !=0)
    {
 /*
      FAST_DIV_NUMERATOR = difference;  // original
      FAST_DIV_DENOMINATOR = sum>>8;
*/

      FAST_DIV_NUMERATOR = difference<<2;
      FAST_DIV_DENOMINATOR = sum>>8;

      FAST_DIV_CTRL = 0x01; // start division

      while ((FAST_DIV_STAT&0x01) == 0x00); // wait for division to complete

      ratio = FAST_DIV_QUOTIENT;
    }

   /* Accumulate the value of P channel */
   FIR_AccadcValue[0] = ratio;
   /* Accumulate the value of T channel */
   FIR_AccadcValue[1] += ADC3_PTAT_DATA;

   /* Update DAC with previous value */
   if (FaultDiag == 0)
   {
    //   DAC_REG0 = DAC_Value;
       if ((nFlags1 & 2) == 0){                     // using ratio as the measured value, if enabled
  //         test = nGain*(ratio+nOffset);            // scale and shift the output
           test = (US) ((Gain * (float) ratio) + Offset);
           DAC_REG0 = test;                         // output value to the DAC
           // DAC_REG0 = 16383;
       }
   }
   else if (FaultDiag == 1)
   {
       DAC_REG0 = 0x0000;
   }
   else if (FaultDiag == 2)
   {
       DAC_REG0 = 0x0FFF;
   }

   /*
    * Application specific Pcoefficient Calculation
    * Dn = H + G * PADC/2^14 + N * (PADC/2^14)^2
    * DAC = Dn
    * Please refer compensation algorithm document.
    */
   coeffP = (NtempCoeff[VarI] * FIR_AccadcValue[0]);
   coeffP = (coeffP >> 14);
   coeffP = coeffP + GtempCoeff[VarI];
   coeffP = (coeffP * FIR_AccadcValue[0]);
   coeffP = (coeffP >> 14);
   coeffP = coeffP + HtempCoeff[VarI];
   DAC_Value = (S2)(coeffP);

   vi = ADC3_VI_DATA;
   ptat = ADC3_PTAT_DATA;
   coil3 = ADC3_COIL3_DATA;
}

/* ========================================================================== */
/**
 * @fn DEMOD1_BPF_config() Configure BPF of DEMOD1.
 *
 * @see pga970_adc.h
 */
/* ========================================================================== */
void DEMOD1_BPF_config(UL B1config, UL A2config, UL A3config)
{
    DEMOD1_BPF_B1 = B1config;
    DEMOD1_BPF_A2 = A2config;
    DEMOD1_BPF_A3 = A3config;
}

/* ========================================================================== */
/**
 * @fn DEMOD1_LPF_config() Configure LPF of DEMOD1.
 *
 * @see pga970_adc.h
 */
/* ========================================================================== */
void DEMOD1_LPF_config(S2 B1config, S2 A2config)
{
    DEMOD1_LPF_B1 = B1config;
    DEMOD1_LPF_A2 = A2config;
}

/* ========================================================================== */
/**
 * @fn DEMOD2_BPF_config() Configure BPF of DEMOD2.
 *
 * @see pga970_adc.h
 */
/* ========================================================================== */
void DEMOD2_BPF_config(UL B1config, UL A2config, UL A3config)
{
    DEMOD2_BPF_B1 = B1config;
    DEMOD2_BPF_A2 = A2config;
    DEMOD2_BPF_A3 = A3config;
}

/* ========================================================================== */
/**
 * @fn DEMOD2_LPF_config() Configure LPF of DEMOD2.
 *
 * @see pga970_adc.h
 */
/* ========================================================================== */
void DEMOD2_LPF_config(S2 B1config, S2 A2config)
{
    DEMOD2_LPF_B1 = B1config;
    DEMOD2_LPF_A2 = A2config;
}

/* ========================================================================== */
/**
 * @fn ADC3_VI_LPF_Config() Configure LPF of VI of ADC3.
 *
 * @see pga970_adc.h
 */
/* ========================================================================== */
void ADC3_VI_LPF_Config(S2 B1config, S2 A2config)
{
    ADC3_VI_LPF_B1 = B1config;
    ADC3_VI_LPF_A2 = A2config;
}

/* ========================================================================== */
/**
 * @fn ADC3_PTAT_LPF_Config() Configure LPF of PTAT of ADC3.
 *
 * @see pga970_adc.h
 */
/* ========================================================================== */
void ADC3_PTAT_LPF_Config(S2 B1config, S2 A2config)
{
    ADC3_PTAT_LPF_B1 = B1config;
    ADC3_PTAT_LPF_A2 = A2config;
}
