/* =============================================================================
*                 Texas Instruments PGA970 Platform Software
* (c) Copyright Texas Instruments, Incorporated. All Rights Reserved.
*
* Use of this software is controlled by the terms and conditions found
* in the license agreement under which this software has been supplied.
* =========================================================================== */
/**
* @file pga970_fram.c
*
* This File contains code used for PGA790 FRAM DATA related activities.
*
* @path $(CSLPATH)\PGA970\source\chip
*
* @rev 1.1
*/
/* -----------------------------------------------------------------------------
*!
*! Revision History
*! ===================================
*! 23-Apr-2015 mf: Modified TusharB.Updated FRAM related functions.
* =========================================================================== */

/*******************************************************************************
* INCLUDE FILES
*******************************************************************************/
#include "pga970_platf.h"

/*
 * The constants needed for T coefficient calculation is read from FRAM DATA
 * and stored into the following variables
 */
VS2 FRAM_DATA_H0_Integer;
VS2 FRAM_DATA_G0_Integer;
VS2 FRAM_DATA_N0_Integer;

VS2 FRAM_DATA_H1_Integer;
VS2 FRAM_DATA_G1_Integer;
VS2 FRAM_DATA_N1_Integer;

VS2 FRAM_DATA_H2_Integer;
VS2 FRAM_DATA_G2_Integer;
VS2 FRAM_DATA_N2_Integer;


#if (FRAM_TEXT_CHECKSUM_CALC==1)
#pragma DATA_SECTION(FRAM_TEXT_ChecksumValue, ".FRAM_TEXT_checksum");
//const UL FRAM_TEXT_ChecksumValue = (0xB5B3EF58);
const UL FRAM_TEXT_ChecksumValue = (0x00000000);
VUL FRAM_TEXT_CalchecksumValue = 0;
UL *fram_text_instAddress = (UL *)FRAM_TEXT_START_ADDRESS;
#endif /*#if (FRAM_TEXT_CHECKSUM_CALC==1)*/

#if(FRAM_DATA_CHECKSUM_CALC==1)
VUL FRAM_DATA_CalchecksumValue = 0;
UL *fram_data_instAddress = (UL *)FRAM_DATA_START_ADDRESS;
#endif /*#if(FRAM_DATA_CHECKSUM_CALC==1)*/

/* ========================================================================== */
/**
 * @fn FRAM_DATA_Read_Coeff() Read coefficients from FRAM DATA location.
 *
 * @see pga970_fram.h
 */
/* ========================================================================== */
void FRAM_DATA_Read_Coeff(void)
{
#if (FRAM_DATA_TESTING == 1)
   S2 *rdPtr;
   rdPtr = (S2 *)(FRAM_DATA_START_ADDRESS);
   FRAM_DATA_H0_Integer = *rdPtr;
   rdPtr++;
   FRAM_DATA_G0_Integer = *rdPtr;
   rdPtr++;
   FRAM_DATA_N0_Integer = *rdPtr;
   rdPtr++;
   FRAM_DATA_H1_Integer = *rdPtr;
   rdPtr++;
   FRAM_DATA_G1_Integer = *rdPtr;
   rdPtr++;
   FRAM_DATA_N1_Integer = *rdPtr;
   rdPtr++;
   FRAM_DATA_H2_Integer = *rdPtr;
   rdPtr++;
   FRAM_DATA_G2_Integer = *rdPtr;
   rdPtr++;
   FRAM_DATA_N2_Integer = *rdPtr;
#else
   FRAM_DATA_H0_Integer = 79;
   FRAM_DATA_G0_Integer = 11336;
   FRAM_DATA_N0_Integer = -53;

   FRAM_DATA_H1_Integer = 7;
   FRAM_DATA_G1_Integer = -22832;
   FRAM_DATA_N1_Integer = 161;

   FRAM_DATA_H2_Integer = -6;
   FRAM_DATA_G2_Integer = 18349;
   FRAM_DATA_N2_Integer = -135;
#endif
}

#if (FRAM_DATA_TESTING == 1)
/* ========================================================================== */
/**
 * @fn FRAM_DATA_Read() Read values from FRAM_DATA.
 *
 * @see pga970_fram.h
 */
/* ========================================================================== */
UC FRAM_DATA_Read(US numBytes, US framdataOffset, void *destPtr)
{
  if((FRAM_DATA_START_ADDRESS + framdataOffset + numBytes)
          <= FRAM_DATA_END_ADDRESS )
  {
     memcpy((void *)destPtr,
            (void *)(FRAM_DATA_START_ADDRESS + framdataOffset),
            numBytes);
     return 1;
  }
  else
  {
     return 0;
  }
}

/* ========================================================================== */
/**
 * @fn FRAM_DATA_Write() Writes to FRAM_DATA Area.
 *
 * @see pga970_fram.h
 */
/* ========================================================================== */
UC FRAM_DATA_Write(US numBytes, US framdataOffset, void *srcPtr)
{
   if((FRAM_DATA_START_ADDRESS + framdataOffset + numBytes)
           <= FRAM_DATA_END_ADDRESS )
   {
      memcpy((void *)(FRAM_DATA_START_ADDRESS + framdataOffset),
             (void *)srcPtr,
             numBytes);
      return 1;
   }
   else
   {
      return 0;
   }

}
#endif /*#if (FRAM_DATA_TESTING == 1)*/

#if (FRAM_TEXT_CHECKSUM_CALC==1)
/* ========================================================================== */
/**
 * @fn FRAM_TEXT_Checksum() Computes FRAM TEXT area Checksum.
 *
 * @see pga970_fram.h
 */
/* ========================================================================== */
#pragma FUNC_ALWAYS_INLINE(FRAM_TEXT_Checksum);
void FRAM_TEXT_Checksum(void)
{
   /*Checksum calculation for FRAM TEXT area*/
   if ((UL)fram_text_instAddress <= FRAM_TEXT_END_ADDRESS)
   {

       /* Read Contents of FRAM_TEXT pointed by fram_text_instAddress*/
       FRAM_TEXT_CalchecksumValue += *fram_text_instAddress;
       fram_text_instAddress++; /* Go to Next Address */
   }
   else
   {
       /* Check if checksum is good or not */
       if(FRAM_TEXT_CalchecksumValue != 0)
       {
           #if (WATCH_DOG_TIMER_TESTING == 1)
             WDT_DISABLE();
           #endif
           /* checksum is not good, reset micro */
           AMUX_CTRL &= (~DAC_OP_TO_DAC_GAIN); /* Disable DAC */
           MICRO_INTERFACE_CONTROL = 0x03;  /* Reset Micro */
       }
       else
       {
           /* checksum is good */
           fram_text_instAddress = (UL *)FRAM_TEXT_START_ADDRESS;
       }
   }
}
#endif /*#if (FRAM_TEXT_CHECKSUM_CALC==1)*/


#if (FRAM_DATA_CHECKSUM_CALC==1)
/* ========================================================================== */
/**
 * @fn FRAM_DATA_Checksum() Computes FRAM DATA area Checksum.
 *
 * @see pga970_fram.h
 */
/* ========================================================================== */
#pragma FUNC_ALWAYS_INLINE(FRAM_DATA_Checksum);
void FRAM_DATA_Checksum(void)
{
   /*Checksum calculation for FRAM DATA area*/
   if ((UL)fram_data_instAddress <= FRAM_DATA_END_ADDRESS)
   {

       /* Read Contents of FRAM_DATA pointed by fram_data_instAddress*/
       FRAM_DATA_CalchecksumValue += *fram_data_instAddress;
       fram_data_instAddress++; /* Go to Next Address */
   }
   else
   {
       /* Check if checksum is good or not */
       if(FRAM_DATA_CalchecksumValue != 0)
       {
           #if (WATCH_DOG_TIMER_TESTING == 1)
             WDT_DISABLE();
           #endif
           /* checksum is not good, reset micro */
           AMUX_CTRL &= (~DAC_OP_TO_DAC_GAIN); /* Disable DAC */
           MICRO_INTERFACE_CONTROL = 0x03;  /* Reset Micro */
       }
       else
       {
           /* checksum is good */
           fram_data_instAddress = (UL *)FRAM_DATA_START_ADDRESS;
       }
   }
}
#endif /*#if (FRAM_DATA_CHECKSUM_CALC==1)*/
