/* =============================================================================
*                 Texas Instruments PGA970 Platform Software
* (c) Copyright Texas Instruments, Incorporated. All Rights Reserved.
*
* Use of this software is controlled by the terms and conditions found
* in the license agreement under which this software has been supplied.
* =========================================================================== */
/**
* @file pga970_app.c
*
* This File contains PGA970 application specific functions.
*
* @path $(CSLPATH)\PGA970\source\oem
*
* @rev 1.3
*/
/* -----------------------------------------------------------------------------
*!
*! Revision History
*! ===================================
*! 23-April-2015 mf: Modified AbhimanyuK.
*! 05-08-20 NJH Changing to different button speed control system
*!
* =========================================================================== */

/*******************************************************************************
* INCLUDE FILES
*******************************************************************************/
#include "pga970_platf.h"
#include "pga970_adc.h"

/* 64 samples -->64*128us = 8.192ms, FIR filter divide by 64 i.e.shift by 6
 * only 16bit  has to be read from ADC register so shift 24bit value by 8*/
#define TADCSHIFT (6 + 8)

#if(WATCH_DOG_TIMER_TESTING==1)
VUS WDOG_Low_Time_Cnt1;
VUS WDOG_Low_Time_Cnt2;
US WDOG_Trig_Low_Timeout;
#endif /*#if(WATCH_DOG_TIMER_TESTING==1)*/

/* State machine variable */
UL StateCoeff;
SL HtempCoeff[2]; /* Calculated H Temperature coefficient value */
SL GtempCoeff[2]; /* Calculated G Temperature coefficient value */
SL NtempCoeff[2]; /* Calculated N Temperature coefficient value */
VUC VarI;         /* ping pong buffer index */
VUC VarJ;         /* ping pong buffer index */
UC varK;
UC FaultDiag;
SL HoldTadcValue;
US Coefficients_Calc_Sampling_Rate;

/* Nigel's constants and variables */
#define Debounce_Counts 50  // 50 x 2ms of the same value
#define Fast_Flash 20
#define Slow_Flash 80
#define Gain_Store 0        // Offset points to the first 4 bytes of the FRAM data store
#define Offset_Store 4      // Offset points to the second 4 bytes of the FRAM data store
#define DAC_Max 0x3fff
#define DAC_Min 0x100
#define Same_Button_Timeout 375 // Approximately 3 seconds before speedup of current setting ?NJH

UI nTime;
UC GPIO_DIR_value;
UC GPIO_Outtype_value;
UC GPIO_in_value;
UC GPIOinTemp;
UC Button_Count;
UC Button_Old;
UC Last_Button;             // ?NJH
US Same_Button_Time = 0;    // ?NJH
UC Debounced_Buttons;
UC New_Button = 0;
UC Setup_State_Machine = 0;
UC Flash_Mode = 0;
UC Flash_Speed = Slow_Flash;
UC SECLOCK_test = 0;
UC nFlags1 = 0;                     // Byte of flags
#define Alternate       (1 << 0)    // bit 0 is used for alternating flash
#define nDAC            (1 << 1)    // bit 1 is used for controlling updating of the DAC
#define Button_Mode     (1 << 2)    // bit 2 is used to control switching between detection of a new button and continuous for up and down adjustment
#define nInit           (1 << 3)    // bit 3 is used to record if the Gain and Offset have been read from memory. set to 1 after read.
#define Fast_Button     (1 << 4)    // bit 4 is used control the speed at which the current setting is incremented or decremented during setup, 1 is faster ?NJH
#define Unlock          (1 << 5)    // bit 5 is used to indicate code unlocked state, 0 = locked
#define Buttons_Up      (1 << 6)    // bit 6 is used to indicate that both buttons have been detected as not pressed for inter entry detection

US ButtonTimer = 0;
#define ButttonTimeout  800         // Used for current setting timeout. 1.6 seconds at 2ms/loop
#define Fast_speed  7               // 7 times the inc/dec speed

US Unlock_timeout = 0;
#define Unlocktime 7500             // approximately 60 seconds
US Button_Sequence = 0;             // 16 bit store for capturing the button press sequence
#define Unlock_Pattern 0b10001000100100 // "David" in Morse code with 0 = dot and 1 = dash "BAA AB AAAB AA BAA"= = 100 01 0001 00 100". 14 bits
UC Button_Pattern = 0;

FLOAT test1 = 0;
FLOAT test2 = 0;
FLOAT test3;
FLOAT Gain = 20;                         // floating point gain value between LVDT and current output
FLOAT Offset = 0;                       // floating point gain value between LVDT and current output
SL Position_1;
SL Current_1;
SL Position_2;
SL Current_2;

union {
    float m_float;
    US  m_word[((sizeof(float))/2)];
    char m_byte[sizeof(float)];
} floatArray;



/* Nigel's functions*/

void Lock_Prcessor(){
    SECLOCK = 0xAA;                                                             // Lock FRAM
    MICRO_INTERFACE_CONTROL = MICRO_INTERFACE_CONTROL | DEBUG_LOCK;             // Lock debug
    nFlags1 = nFlags1 & ~Unlock;                                                // set as locked
    Flash_Mode = 0;                                                             // clear unlock flash mode
}

void Update_Unlock_Sequence(){
    switch (Button_Pattern){
        case 0b00:                                                              // if button pattern is both pressed then clear the recorded pattern (pattern 0b00)
            Button_Sequence = 0;
            nFlags1 = (nFlags1 & ~Buttons_Up);                                  // bitwise and with the inverse pattern to clear the bit, clear the un-pressed time detection
            break;

        case 0b10:                                                              // if button "A" is pressed then shift the sequence and record a "0"
            if (nFlags1 && Buttons_Up == Buttons_Up){
                Button_Sequence = ((Button_Sequence << 1) & 0b0011111111111110);    // shift the sequence left and ensure that the lowest bit is zero
                nFlags1 = (nFlags1 & ~Buttons_Up);                              // bitwise and with the inverse pattern to clear the bit, clear the un-pressed time detection
            }
            break;

        case 0b01:                                                              // if button "B" is pressed then shift the sequence and record a "1"

            if (nFlags1 && Buttons_Up == Buttons_Up){
                Button_Sequence = (((Button_Sequence << 1) & 0b0011111111111110)+1);    // shift the sequence left and ensure that the lowest bit is zero then add 1
                nFlags1 = (nFlags1 & ~Buttons_Up);                              // bitwise and with the inverse pattern to clear the bit, clear the un-pressed time detection
            }
            break;

        case 0b11:                                                              // if button pattern is neither pressed then don't update
            nFlags1 = (nFlags1 | Buttons_Up);                                   // indicate time between button presses
            break;

        default:
            break;
    }
    if (Button_Sequence == Unlock_Pattern){                                     // check unlock sequence and unlock if valid
        Button_Sequence = 0;                                                    // clear the captured sequence sequence
        nFlags1 = nFlags1 | Unlock;                                             // set as unlocked
        Flash_Mode = 8;                                                         // unlock flash mode
        Unlock_timeout = Unlocktime;                                            // unlock timeout
        SECLOCK = 0x00;                                                         // unlock FRAM
        MICRO_INTERFACE_CONTROL = MICRO_INTERFACE_CONTROL & !DEBUG_LOCK;        // unlock debug
    }
}

void ButtonDebounce(){
    GPIO_DIR_value = GPIO_DIR;                                                  // store the GPIO direction setting
    GPIO_Outtype_value = Get_GPIO_Outtype();                                    // Store the GPIO output types
    GPIO_Config(GPIO1_DIR_INPUT | GPIO2_DIR_INPUT , GPIO1_PUSH_PULL | GPIO2_PUSH_PULL);    // Switch both GPIO to input and push-pull
    GPIOinTemp = GPIO_DIR;
    GPIO_in_value = GPIO_INPUT;                                                 // Get the value of GPIO
    GPIO_Config(GPIO_DIR_value , GPIO_Outtype_value);                           // restore the original GPIO configuration
    GPIOinTemp = GPIO_DIR;
    if (GPIO_in_value == Button_Old) {                                          // Debounce the buttons
        if (Button_Count < Debounce_Counts) {
            Button_Count++;                                                     // increment button debounce count
            if (Button_Count >= Debounce_Counts) {
                New_Button = 1;                                                 // flag that there is a new button value
                Debounced_Buttons = GPIO_in_value;                              // store the new value
                Button_Pattern = Debounced_Buttons;
                Update_Unlock_Sequence();                                       //
            }
        }
    }
    else {
        Button_Count = 0;                                                       // reset the debounce counter
        Button_Old = GPIO_in_value;                                             // store the latest value from the buttons
    }
}

void DAQ_up(){
    US DAC_Value = DAC_REG0;                        // increment the DAC value
    if ((nFlags1 & Fast_Button) > 0) DAC_Value = DAC_Value + Fast_speed;
        else DAC_Value++;                           // check which speed to increment
    if (DAC_Value > DAC_Max) DAC_REG0 = DAC_Max;    // check limits
    else DAC_REG0 = DAC_Value;
    ButtonTimer = 0;                                // zero the timeout
}

void DAQ_down(){
    US DAC_Value = DAC_REG0;                        // decrement the DAC value
    if ((nFlags1 & Fast_Button) > 0) DAC_Value = DAC_Value - Fast_speed;
    else DAC_Value--;                               // check which speed to decrement
    if (DAC_Value < DAC_Min) DAC_REG0 = DAC_Min;    // check limits
    else DAC_REG0 = DAC_Value;
    ButtonTimer = 0;                                // zero the timeout
}

void Continuous_button(){
    if (Last_Button == Debounced_Buttons){
        if (Same_Button_Time > Same_Button_Timeout){
            nFlags1 = (nFlags1 | Fast_Button);     // flag faster increment/decrement
        }
        else Same_Button_Time++;                    // increment timer
    }
    else {
        Same_Button_Time = 0;
        nFlags1 = (nFlags1 & ~Fast_Button);        // flag slow increment/decrement
        Last_Button = Debounced_Buttons;
    }
}

void NDATA_Float_Write(UC offset, FLOAT value){     // Writes 4 bytes to the offset position in NDATA
    floatArray.m_float = value;                     // put value into the union
    S2 *wtPtr;                                      // create a pointer
    wtPtr = (S2 *) (NDATA_START_ADDRESS + offset);  // point at the correct position in the FRAM
    int x = 0;                                      // write the 2 words into FRAM
    while (x < 2){
        *wtPtr = floatArray.m_word[x];
        x++;
    }
}

FLOAT NDATA_Float_Read(UC offset){                  // Gets a FLOAT from the offset position in NDATA
    S2 *rdPtr;                                      // create a pointer
    rdPtr = (S2 *) (NDATA_START_ADDRESS + offset);  // point at the correct position in the FRAM
    int x = 0;                                      // read the 2 words from FRAM
    while (x < 2){
        floatArray.m_word[x] = *rdPtr;
        x++;
    }
    return floatArray.m_float;
}

void Init_check(){                                      // if gain and offset have not been initialised then get them from FRAM
    if ((nFlags1 & nInit) == 0) {
        Lock_Prcessor();                                // lock processor on first time around
        Gain = NDATA_Float_Read(Gain_Store);            // Get Gain
        Offset = NDATA_Float_Read(Offset_Store);        // Get Offset
        nFlags1 = (nFlags1 | nInit);                    // Flag done
    }
}

void Unlock_check(){
    if (Unlock_timeout > 0) {                           // check for unlocked
        Unlock_timeout--;                               // count down timeout
        if (Unlock_timeout == 0) {                      // if now zero then lock
            Lock_Prcessor();                            // lock code here including reseting of the unlock flash mode
        }
    }
}

void Buttons_state_machine(){                           // Buttons state machine
    if ((New_Button == 1) | ((nFlags1 & Button_Mode) == Button_Mode)){  // check for a button pressed
        New_Button = 0;                                 // clear the new button flag
        switch (Setup_State_Machine){
           case 0 :                                     // if the state machine is "0" (normal run mode)
               if (Debounced_Buttons == 0b00){          //   then check for buttons = b00, both pressed indicating user input requesting setup mode
                   Flash_Mode = 1;                      // turn on flashing of LED1  to indicate that the user should set the first LVDT position
                   Setup_State_Machine = 1;             // next state
               }
               break;

           case 1 :                                     // if the state machine is "1"
               if (Debounced_Buttons == 0b10){          //   then check for buttons = b10, button 1 pressed indicating that the first position of the LVDT has been set
                   Position_1 = ratio;                  // Store the LVDT position. moved this line from case 0 NJH 05-08-20
                   Flash_Mode = 5;                      // turn on synchronous fast flashing of both LEDs
                   GPIO1_LOW();                          // ensure both LEDs are in the same state
                   GPIO2_LOW();
                   nFlags1 = (nFlags1 | nDAC);          // turn off updating of DAC
                   nFlags1 = (nFlags1 | Button_Mode);   // Switch to continuous button control up and down current
                   Setup_State_Machine = 2;             // next state
                }
                break;

           case 2 :                                     //  if the state machine is "2" current up and down on buttons for first current setting
               Continuous_button();                     // check for fast button speed
               if (ButtonTimer++ < ButttonTimeout){     // increment the timeout and control the up down loop
                   if (Debounced_Buttons == 0b01) DAQ_up();     // If button 2 is pressed
                   if (Debounced_Buttons == 0b10) DAQ_down();   // If button 1 is pressed
               }
               else {                                   // if the timeout has been exceeded then:
                   Current_1 = DAC_REG0;                // store first DAC value
                   nFlags1 = (nFlags1 & ~Button_Mode);  // re enable new button check
                   ButtonTimer = 0;                     // zero the timeout
                   Flash_Mode = 3;                      // turn on slow flash of LED2
                   nFlags1 = (nFlags1 & ~Fast_Button);  // Flag slow buttons (should not realy be required NJH 5-008-20)
                   Setup_State_Machine = 3;
               }
               break;

           case 3:                                      // wait for button 2 press indicating that the operator has moved the LVDT to the second position.
               if (Debounced_Buttons == 0b01){
                   Position_2 = ratio;                  // Store the current LVDT position
                   Flash_Mode = 6;                      // turn on synchronous fast flashing of both LEDs indicating current set mode
                   nFlags1 = (nFlags1 | nDAC);          // turn off updating of DAC
                   nFlags1 = (nFlags1 | Button_Mode);   // Switch to continuous button control up and down current
                   Setup_State_Machine = 4;             // next state
               }
               break;

           case 4 :                                     // current up and down on buttons for first current setting
               Continuous_button();                     // check for fast button speed
               if (ButtonTimer++ < ButttonTimeout){     // increment the timeout and control the up down loop
                   if (Debounced_Buttons == 0b01) DAQ_up();     // If button 2 is pressed
                   if (Debounced_Buttons == 0b10) DAQ_down();   // If button 1 is pressed
               }
               else {
                   Current_2 = DAC_REG0;                // store second DAC value
                   ButtonTimer = 0;                     // zero the timeout
                   Flash_Mode = 0;                      // turn off LED
                   nFlags1 = (nFlags1 & ~Fast_Button);  // Flag slow buttons
                   Setup_State_Machine = 5;             // next state, do the calculations and store the results..
               }
               break;

           case 5:                                      // calculate gain
               Gain = (float)(Current_2 - Current_1)/(float)(Position_2 - Position_1);
               Setup_State_Machine = 6;                 // next state
               break;

           case 6:                                      // calculate offset
               Offset = (float) Current_1 - (float) (Gain * Position_1);
               Setup_State_Machine = 7;                 // next state
               break;

           case 7:                                      // store gain
               NDATA_Float_Write(Gain_Store, Gain);
               Setup_State_Machine = 8;                 // next state
               break;

           case 8:                                      // store offset, re-enable the normal operation modes
               NDATA_Float_Write(Offset_Store, Offset);
               nFlags1 = (nFlags1 & ~Button_Mode);      // re enable new button check
               nFlags1 = (nFlags1 & ~nDAC);             // re-enable LVDT DAC output
               Setup_State_Machine = 0;
               break;

           default:
               Flash_Mode = 0;                          // turn off flashing of LEDs
               Setup_State_Machine = 0;
               break;
        }
    }
}

void LED_State_Machine(){                              // LEDs state machine
    nTime++;
    if (nTime > Flash_Speed){
        nTime = 0;
        switch (Flash_Mode){
            case 0 : {                                 // turn off LEDs
                GPIO1_LOW();
                GPIO2_LOW();
            }
            break;

            case 1 :                                   // LED 1 slow flash
                Flash_Speed = Slow_Flash;
                GPIO1_TOGGLE();
                GPIO2_LOW();
            break;

            case 2 :                                   // LED 1 fast flash
                Flash_Speed = Fast_Flash;
                GPIO1_TOGGLE();
                GPIO2_LOW();
            break;

            case 3 :                                   // LED 2 slow flash
                Flash_Speed = Slow_Flash;
                GPIO2_TOGGLE();
                GPIO1_LOW();
            break;

            case 4 :                                   // LED 2 fast flash
                Flash_Speed = Fast_Flash;
                GPIO2_TOGGLE();
                GPIO1_LOW();
            break;

            case 5 :                                   // Both LEDs synchronous fast flash
                Flash_Speed = Fast_Flash;              // make sure both LEDs are in the same state when switching to this state
                GPIO1_TOGGLE();
                GPIO2_TOGGLE();
            break;

            case 6 :                                   // Both LEDs alternate fast flash
                Flash_Speed = Fast_Flash;              // set the flash speed
                if ((nFlags1 & Alternate) == 1) {      // Check the alternating state
                    GPIO1_HIGH();
                    GPIO2_LOW();
                    nFlags1 = (nFlags1 & 0xFE);        // flip the alternating state
                }
                else {
                    GPIO1_LOW();
                    GPIO2_HIGH();
                    nFlags1 = (nFlags1 | 0x01);        // flip the alternating state
                }

            break;

            case 7 :                                   // error indication
                GPIO1_HIGH();
                GPIO2_HIGH();
                break;

            case 8 :                                   // unlock indication part 1
                Flash_Speed = Slow_Flash;
                GPIO1_LOW();
                GPIO2_LOW();
                Flash_Mode = 9;
                break;

            case 9:                                    // unlock indication part 2
                GPIO1_HIGH();                          // slow flash on LED1
                GPIO2_LOW();
                Flash_Mode = 10;
                break;

            case 10:                                   // unlock indication part 3
                GPIO1_LOW();
                GPIO2_LOW();
                Flash_Mode = 11;
                break;

            case 11:                                   // unlock indication part 4
                Flash_Speed = Fast_Flash;              // set the flash speed
                GPIO1_LOW();
                GPIO2_HIGH();                          // fast flash on LED2
                Flash_Mode = 8;
                break;

            default:
                break;
          }
    }
}



/* ========================================================================== */
/**
 * (*CaseFunctions[4]) Function pointer points to state machine functions.
 *
 * @param none
 *
 * @return none
 *
 * @post calculated T coefficients into appropriate variables.
 *
 * @see none
 */
/* ========================================================================== */

void (*CaseFunctions[4])(void);

/* ========================================================================== */
/**
 * CaseX() State machine, where X is ranging from 0 to 3.
 *         calculates T coefficients.
 * @param none
 *
 * @return none
 *
 * @post calculated T coefficients stored into appropriate variables
 *
 * @see none
 */
/* ========================================================================== */
void Case0(void)
{
    /* Read the filtered value Tadc */
    HoldTadcValue = (FIR_AccadcValue[1] >> TADCSHIFT);
    FIR_AccadcValue[1] = 0;
    /* H Temperature Coefficient Calculation
     * H = h0ee/2^14 + (h1ee/2^14) * TADC/2^14 + ((h2ee/2^14) * ((TADC/2^14)^2))
     * Please refer compensation algorithm document.
     */
    HtempCoeff[VarJ] = FRAM_DATA_H2_Integer * HoldTadcValue;
    HtempCoeff[VarJ] = HtempCoeff[VarJ] >> 14;
    HtempCoeff[VarJ] = HtempCoeff[VarJ] + FRAM_DATA_H1_Integer;
    HtempCoeff[VarJ] = HtempCoeff[VarJ] * HoldTadcValue;
    HtempCoeff[VarJ] = HtempCoeff[VarJ] >> 14;
    HtempCoeff[VarJ] = HtempCoeff[VarJ] + FRAM_DATA_H0_Integer;
}

void Case1(void)
{
    /*
     * G Temperature Coefficient Calculation
     * G = g0ee/2^14 + (g1ee/2^14) * TADC/2^14 + ((g2ee/2^14) * ((TADC/2^14)^2))
     * Please refer compensation algorithm document.
     */
    GtempCoeff[VarJ] = FRAM_DATA_G2_Integer * HoldTadcValue;
    GtempCoeff[VarJ] = GtempCoeff[VarJ] >> 14;
    GtempCoeff[VarJ] = GtempCoeff[VarJ] + FRAM_DATA_G1_Integer;
    GtempCoeff[VarJ] = GtempCoeff[VarJ] * HoldTadcValue;
    GtempCoeff[VarJ] = GtempCoeff[VarJ] >> 14;
    GtempCoeff[VarJ] = GtempCoeff[VarJ] + FRAM_DATA_G0_Integer;
}

void Case2(void)
{
    /*
     * N Temperature Coefficient Calculation
     * N = n0ee/2^14 + (n1ee/2^14) * TADC/2^14 + ((n2ee/2^14) * ((TADC/2^14)^2))
     * Please refer compensation algorithm document.
     */
    NtempCoeff[VarJ] = FRAM_DATA_N2_Integer * HoldTadcValue;
    NtempCoeff[VarJ] = NtempCoeff[VarJ] >> 14;
    NtempCoeff[VarJ] = NtempCoeff[VarJ] + FRAM_DATA_N1_Integer;
    NtempCoeff[VarJ] = NtempCoeff[VarJ] * HoldTadcValue;
    NtempCoeff[VarJ] = NtempCoeff[VarJ] >> 14;
    NtempCoeff[VarJ] = NtempCoeff[VarJ] + FRAM_DATA_N0_Integer;
#if (FRAM_TEXT_CHECKSUM_CALC==1)
    FRAM_TEXT_Checksum();
#endif

#if (FRAM_DATA_CHECKSUM_CALC==1)
    FRAM_DATA_Checksum();
#endif
}

void Case3(void)    // application software here
{
    Init_check();                                     // If gain and offset have not been initialised then get them from FRAM. If unlocked then lock it
    Unlock_check();                                   // check for unlocked
    if ((nFlags1 & Unlock) == 0){                     // inhibit buttons while in unlocked mode
        ButtonDebounce();                             // Button debounce
        Buttons_state_machine();                      // Buttons state machine
    }
    LED_State_Machine();                              // LEDs state machine

#if (DIAGNOSTIC_TESTING == 1)
    /* Read PSMON flag */
    varK = PSMON1;
       /* Clear PSMON1 register */
       if(PSMON1 | PSMON2)
       {
           PSMON1 = 0xFF; /* Write 1 to clear it */
           PSMON2 = 0x1F; /* Write 1 to clear it */
           /* True fault flag */
           FaultDiag = 0x01;
       }

       /* Read AFEDIAG flag */
       varK = AFEDIAG;
       /* Clear AFEDIAG register */
       if(AFEDIAG)
       {
           AFEDIAG = 0xFF; /* Write 1 to clear it */
           /* True fault flag */
           FaultDiag = 0x02;
       }

       /* Switch Ping Pong Buffer */
       VarI ^= 1;
       VarJ ^= 1;
#endif /*#if (DIAGNOSTIC_TESTING == 1)*/
       /* Re-initialize watchdog time-out value */
#if (WATCH_DOG_TIMER_TESTING == 1)
    WDOG_Low_Time_Cnt2 = ADC_Count1;
    if(WDOG_Low_Time_Cnt2 < WDOG_Low_Time_Cnt1)
    {
        WDOG_Low_Time_Cnt2 = WDOG_Low_Time_Cnt2 + (ADC_COUNT1_16BIT_MAX_VAL - WDOG_Low_Time_Cnt1);
    }
    else
    {
        WDOG_Low_Time_Cnt2 = (WDOG_Low_Time_Cnt2 - WDOG_Low_Time_Cnt1);
    }
    if(WDOG_Low_Time_Cnt2 > WDOG_Trig_Low_Timeout)
    {
        WDT_TRIGGER(APP_WATCHDOG_TIMEOUT);
        WDOG_Low_Time_Cnt1=0;
    }
#endif /*#if (WATCH_DOG_TIMER_TESTING == 1)*/
}
/* ========================================================================== */
/**
 * @fn APP_Reset_Init() Initializes global variables.
 *
 * @see pga970_app.h
 */
/* ========================================================================== */
void APP_Reset_Init(void)
{
    HoldTadcValue = 0;
    HtempCoeff[0] = 0;
    HtempCoeff[1] = 0;
    GtempCoeff[0] = 0;
    GtempCoeff[1] = 0;
    NtempCoeff[0] = 0;
    NtempCoeff[1] = 0;
    StateCoeff = 3;
    VarI = 0;
    VarJ = 1;
    FaultDiag = 0;
    CaseFunctions[0] = Case0;
    CaseFunctions[1] = Case1;
    CaseFunctions[2] = Case2;
    CaseFunctions[3] = Case3;
#if(WATCH_DOG_TIMER_TESTING==1)
    WDOG_Low_Time_Cnt1=0;
    WDOG_Low_Time_Cnt2=0;
#endif /*#if(WATCH_DOG_TIMER_TESTING==1)*/
}
/* ========================================================================== */
/**
 * @fn APP_Calculate_Coeff() State machine for T coefficient & P calculation.
 *
 * @see pga970_app.h
 */
/* ========================================================================== */
void APP_Calculate_Coeff(void)
{
   unsigned int state;

   state = StateCoeff;
   state = state + 1;
   if (state >= 4)
   {
       state = 0;
   }
   StateCoeff = state;

   (*CaseFunctions[state])();
}
